@@ -1,11 +1,13 @@
 from importlib.metadata import version as _version
 
-from xarray import testing, tutorial
+from xarray import groupers, testing, tutorial
 from xarray.backends.api import (
     load_dataarray,
     load_dataset,
     open_dataarray,
     open_dataset,
+    open_datatree,
+    open_groups,
     open_mfdataset,
     save_mfdataset,
 )
@@ -31,15 +33,24 @@ from xarray.core.concat import concat
 from xarray.core.coordinates import Coordinates
 from xarray.core.dataarray import DataArray
 from xarray.core.dataset import Dataset
+from xarray.core.datatree import DataTree
+from xarray.core.datatree_mapping import map_over_datasets
 from xarray.core.extensions import (
     register_dataarray_accessor,
     register_dataset_accessor,
+    register_datatree_accessor,
 )
 from xarray.core.indexes import Index
 from xarray.core.indexing import IndexSelResult
 from xarray.core.merge import Context, MergeError, merge
 from xarray.core.options import get_options, set_options
 from xarray.core.parallel import map_blocks
+from xarray.core.treenode import (
+    InvalidTreeError,
+    NotFoundInTreeError,
+    TreeIsomorphismError,
+    group_subtrees,
+)
 from xarray.core.variable import IndexVariable, Variable, as_variable
 from xarray.namedarray.core import NamedArray
 from xarray.util.print_versions import show_versions
@@ -55,6 +66,7 @@ except Exception:
 # `mypy --strict` running in projects that import xarray.
 __all__ = (
     # Sub-packages
+    "groupers",
     "testing",
     "tutorial",
     # Top-level functions
@@ -75,19 +87,24 @@ __all__ = (
     "cross",
     "full_like",
     "get_options",
+    "group_subtrees",
     "infer_freq",
     "load_dataarray",
     "load_dataset",
     "map_blocks",
+    "map_over_datasets",
     "merge",
     "ones_like",
     "open_dataarray",
     "open_dataset",
+    "open_datatree",
+    "open_groups",
     "open_mfdataset",
     "open_zarr",
     "polyval",
     "register_dataarray_accessor",
     "register_dataset_accessor",
+    "register_datatree_accessor",
     "save_mfdataset",
     "set_options",
     "show_versions",
@@ -100,14 +117,18 @@ __all__ = (
     "Coordinates",
     "DataArray",
     "Dataset",
+    "DataTree",
     "Index",
     "IndexSelResult",
     "IndexVariable",
     "Variable",
     "NamedArray",
     # Exceptions
+    "InvalidTreeError",
     "MergeError",
+    "NotFoundInTreeError",
     "SerializationWarning",
+    "TreeIsomorphismError",
     # Constants
     "__version__",
     "ALL_DIMS",
