@@ -9,8 +9,16 @@ import itertools
 import operator
 import warnings
 from collections import Counter
-from collections.abc import Hashable, Iterable, Iterator, Mapping, Sequence, Set
-from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar, Union, cast, overload
+from collections.abc import (
+    Callable,
+    Hashable,
+    Iterable,
+    Iterator,
+    Mapping,
+    Sequence,
+    Set,
+)
+from typing import TYPE_CHECKING, Any, Literal, TypeVar, Union, cast, overload
 
 import numpy as np
 
@@ -23,7 +31,7 @@ from xarray.core.indexes import Index, filter_indexes_from_coords
 from xarray.core.merge import merge_attrs, merge_coordinates_without_align
 from xarray.core.options import OPTIONS, _get_keep_attrs
 from xarray.core.types import Dims, T_DataArray
-from xarray.core.utils import is_dict_like, is_duck_dask_array, is_scalar, parse_dims
+from xarray.core.utils import is_dict_like, is_scalar, parse_dims_as_set, result_name
 from xarray.core.variable import Variable
 from xarray.namedarray.parallelcompat import get_chunked_array_type
 from xarray.namedarray.pycompat import is_chunked_array
@@ -38,7 +46,6 @@ if TYPE_CHECKING:
     MissingCoreDimOptions = Literal["raise", "copy", "drop"]
 
 _NO_FILL_VALUE = utils.ReprObject("<no-fill-value>")
-_DEFAULT_NAME = utils.ReprObject("<default-name>")
 _JOINS_WITHOUT_FILL_VALUES = frozenset({"inner", "exact"})
 
 
@@ -63,9 +70,9 @@ class _UFuncSignature:
 
     Attributes
     ----------
-    input_core_dims : tuple[tuple]
+    input_core_dims : tuple[tuple, ...]
         Core dimension names on each input variable.
-    output_core_dims : tuple[tuple]
+    output_core_dims : tuple[tuple, ...]
         Core dimension names on each output variable.
     """
 
@@ -133,11 +140,7 @@ class _UFuncSignature:
         return not self == other
 
     def __repr__(self):
-        return "{}({!r}, {!r})".format(
-            type(self).__name__,
-            list(self.input_core_dims),
-            list(self.output_core_dims),
-        )
+        return f"{type(self).__name__}({list(self.input_core_dims)!r}, {list(self.output_core_dims)!r})"
 
     def __str__(self):
         lhs = ",".join("({})".format(",".join(dims)) for dims in self.input_core_dims)
@@ -182,18 +185,6 @@ class _UFuncSignature:
         return str(alt_signature)
 
 
-def result_name(objects: Iterable[Any]) -> Any:
-    # use the same naming heuristics as pandas:
-    # https://github.com/blaze/blaze/issues/458#issuecomment-51936356
-    names = {getattr(obj, "name", _DEFAULT_NAME) for obj in objects}
-    names.discard(_DEFAULT_NAME)
-    if len(names) == 1:
-        (name,) = names
-    else:
-        name = None
-    return name
-
-
 def _get_coords_list(args: Iterable[Any]) -> list[Coordinates]:
     coords_list = []
     for arg in args:
@@ -322,7 +313,7 @@ def apply_dataarray_vfunc(
                 variable, coords=coords, indexes=indexes, name=name, fastpath=True
             )
             for variable, coords, indexes in zip(
-                result_var, result_coords, result_indexes
+                result_var, result_coords, result_indexes, strict=True
             )
         )
     else:
@@ -403,7 +394,7 @@ def _unpack_dict_tuples(
 ) -> tuple[dict[Hashable, Variable], ...]:
     out: tuple[dict[Hashable, Variable], ...] = tuple({} for _ in range(num_outputs))
     for name, values in result_vars.items():
-        for value, results_dict in zip(values, out):
+        for value, results_dict in zip(values, out, strict=True):
             results_dict[name] = value
     return out
 
@@ -418,7 +409,7 @@ def _check_core_dims(signature, variable_args, name):
     """
     missing = []
     for i, (core_dims, variable_arg) in enumerate(
-        zip(signature.input_core_dims, variable_args)
+        zip(signature.input_core_dims, variable_args, strict=True)
     ):
         # Check whether all the dims are on the variable. Note that we need the
         # `hasattr` to check for a dims property, to protect against the case where
@@ -450,7 +441,7 @@ def apply_dict_of_variables_vfunc(
     grouped_by_name = collect_dict_values(args, names, fill_value)
 
     result_vars = {}
-    for name, variable_args in zip(names, grouped_by_name):
+    for name, variable_args in zip(names, grouped_by_name, strict=True):
         core_dim_present = _check_core_dims(signature, variable_args, name)
         if core_dim_present is True:
             result_vars[name] = func(*variable_args)
@@ -542,7 +533,7 @@ def apply_dataset_vfunc(
     if signature.num_outputs > 1:
         out = tuple(
             _fast_dataset(*args)
-            for args in zip(result_vars, list_of_coords, list_of_indexes)
+            for args in zip(result_vars, list_of_coords, list_of_indexes, strict=True)
         )
     else:
         (coord_vars,) = list_of_coords
@@ -612,11 +603,13 @@ def apply_groupby_func(func, *args):
             iterator = itertools.repeat(arg)
         iterators.append(iterator)
 
-    applied: Iterator = (func(*zipped_args) for zipped_args in zip(*iterators))
+    applied: Iterator = (
+        func(*zipped_args) for zipped_args in zip(*iterators, strict=False)
+    )
     applied_example, applied = peek_at(applied)
     combine = first_groupby._combine  # type: ignore[attr-defined]
     if isinstance(applied_example, tuple):
-        combined = tuple(combine(output) for output in zip(*applied))
+        combined = tuple(combine(output) for output in zip(*applied, strict=True))
     else:
         combined = combine(applied)
     return combined
@@ -633,7 +626,7 @@ def unified_dim_sizes(
                 "broadcasting cannot handle duplicate "
                 f"dimensions on a variable: {list(var.dims)}"
             )
-        for dim, size in zip(var.dims, var.shape):
+        for dim, size in zip(var.dims, var.shape, strict=True):
             if dim not in exclude_dims:
                 if dim not in dim_sizes:
                     dim_sizes[dim] = size
@@ -737,7 +730,7 @@ def apply_variable_ufunc(
             if isinstance(arg, Variable)
             else arg
         )
-        for arg, core_dims in zip(args, signature.input_core_dims)
+        for arg, core_dims in zip(args, signature.input_core_dims, strict=True)
     ]
 
     if any(is_chunked_array(array) for array in input_data):
@@ -762,7 +755,7 @@ def apply_variable_ufunc(
             allow_rechunk = dask_gufunc_kwargs.get("allow_rechunk", None)
             if allow_rechunk is None:
                 for n, (data, core_dims) in enumerate(
-                    zip(input_data, signature.input_core_dims)
+                    zip(input_data, signature.input_core_dims, strict=True)
                 ):
                     if is_chunked_array(data):
                         # core dimensions cannot span multiple chunks
@@ -844,7 +837,7 @@ def apply_variable_ufunc(
     )
 
     output: list[Variable] = []
-    for dims, data in zip(output_dims, result_data):
+    for dims, data in zip(output_dims, result_data, strict=True):
         data = as_compatible_data(data)
         if data.ndim != len(dims):
             raise ValueError(
@@ -1068,7 +1061,7 @@ def apply_ufunc(
     supported:
 
     >>> magnitude(3, 4)
-    5.0
+    np.float64(5.0)
     >>> magnitude(3, np.array([0, 4]))
     array([3., 5.])
     >>> magnitude(array, 0)
@@ -1146,6 +1139,8 @@ def apply_ufunc(
     dask.array.apply_gufunc
     xarray.map_blocks
 
+    Notes
+    -----
     :ref:`dask.automatic-parallelization`
         User guide describing :py:func:`apply_ufunc` and :py:func:`map_blocks`.
 
@@ -1589,15 +1584,6 @@ def cross(
     array([-3,  6, -3])
     Dimensions without coordinates: dim_0
 
-    Vector cross-product with 2 dimensions, returns in the perpendicular
-    direction:
-
-    >>> a = xr.DataArray([1, 2])
-    >>> b = xr.DataArray([4, 5])
-    >>> xr.cross(a, b, dim="dim_0")
-    <xarray.DataArray ()> Size: 8B
-    array(-3)
-
     Vector cross-product with 3 dimensions but zeros at the last axis
     yields the same results as with 2 dimensions:
 
@@ -1608,42 +1594,6 @@ def cross(
     array([ 0,  0, -3])
     Dimensions without coordinates: dim_0
 
-    One vector with dimension 2:
-
-    >>> a = xr.DataArray(
-    ...     [1, 2],
-    ...     dims=["cartesian"],
-    ...     coords=dict(cartesian=(["cartesian"], ["x", "y"])),
-    ... )
-    >>> b = xr.DataArray(
-    ...     [4, 5, 6],
-    ...     dims=["cartesian"],
-    ...     coords=dict(cartesian=(["cartesian"], ["x", "y", "z"])),
-    ... )
-    >>> xr.cross(a, b, dim="cartesian")
-    <xarray.DataArray (cartesian: 3)> Size: 24B
-    array([12, -6, -3])
-    Coordinates:
-      * cartesian  (cartesian) <U1 12B 'x' 'y' 'z'
-
-    One vector with dimension 2 but coords in other positions:
-
-    >>> a = xr.DataArray(
-    ...     [1, 2],
-    ...     dims=["cartesian"],
-    ...     coords=dict(cartesian=(["cartesian"], ["x", "z"])),
-    ... )
-    >>> b = xr.DataArray(
-    ...     [4, 5, 6],
-    ...     dims=["cartesian"],
-    ...     coords=dict(cartesian=(["cartesian"], ["x", "y", "z"])),
-    ... )
-    >>> xr.cross(a, b, dim="cartesian")
-    <xarray.DataArray (cartesian: 3)> Size: 24B
-    array([-10,   2,   5])
-    Coordinates:
-      * cartesian  (cartesian) <U1 12B 'x' 'y' 'z'
-
     Multiple vector cross-products. Note that the direction of the
     cross product vector is defined by the right-hand rule:
 
@@ -1740,11 +1690,11 @@ def cross(
             if a.sizes[dim] < b.sizes[dim]:
                 a = a.pad({dim: (0, 1)}, constant_values=0)
                 # TODO: Should pad or apply_ufunc handle correct chunking?
-                a = a.chunk({dim: -1}) if is_duck_dask_array(a.data) else a
+                a = a.chunk({dim: -1}) if is_chunked_array(a.data) else a
             else:
                 b = b.pad({dim: (0, 1)}, constant_values=0)
                 # TODO: Should pad or apply_ufunc handle correct chunking?
-                b = b.chunk({dim: -1}) if is_duck_dask_array(b.data) else b
+                b = b.chunk({dim: -1}) if is_chunked_array(b.data) else b
         else:
             raise ValueError(
                 f"{dim!r} on {'a' if a.sizes[dim] == 1 else 'b'} is incompatible:"
@@ -1861,7 +1811,7 @@ def dot(
     from xarray.core.dataarray import DataArray
     from xarray.core.variable import Variable
 
-    if any(not isinstance(arr, (Variable, DataArray)) for arr in arrays):
+    if any(not isinstance(arr, Variable | DataArray) for arr in arrays):
         raise TypeError(
             "Only xr.DataArray and xr.Variable are supported."
             f"Given {[type(arr) for arr in arrays]}."
@@ -1878,16 +1828,15 @@ def dot(
     einsum_axes = "abcdefghijklmnopqrstuvwxyz"
     dim_map = {d: einsum_axes[i] for i, d in enumerate(all_dims)}
 
+    dot_dims: set[Hashable]
     if dim is None:
         # find dimensions that occur more than once
         dim_counts: Counter = Counter()
         for arr in arrays:
             dim_counts.update(arr.dims)
-        dim = tuple(d for d, c in dim_counts.items() if c > 1)
+        dot_dims = {d for d, c in dim_counts.items() if c > 1}
     else:
-        dim = parse_dims(dim, all_dims=tuple(all_dims))
-
-    dot_dims: set[Hashable] = set(dim)
+        dot_dims = parse_dims_as_set(dim, all_dims=set(all_dims))
 
     # dimensions to be parallelized
     broadcast_dims = common_dims - dot_dims
@@ -2218,7 +2167,7 @@ def _calc_idxminmax(
     # Handle chunked arrays (e.g. dask).
     if is_chunked_array(array.data):
         chunkmanager = get_chunked_array_type(array.data)
-        chunks = dict(zip(array.dims, array.chunks))
+        chunks = dict(zip(array.dims, array.chunks, strict=True))
         dask_coord = chunkmanager.from_array(array[dim].data, chunks=chunks[dim])
         data = dask_coord[duck_array_ops.ravel(indx.data)]
         res = indx.copy(data=duck_array_ops.reshape(data, indx.shape))
@@ -2307,7 +2256,7 @@ def unify_chunks(*objects: Dataset | DataArray) -> tuple[Dataset | DataArray, ..
     _, chunked_data = chunkmanager.unify_chunks(*unify_chunks_args)
     chunked_data_iter = iter(chunked_data)
     out: list[Dataset | DataArray] = []
-    for obj, ds in zip(objects, datasets):
+    for obj, ds in zip(objects, datasets, strict=True):
         for k, v in ds._variables.items():
             if v.chunks is not None:
                 ds._variables[k] = v.copy(data=next(chunked_data_iter))
