@@ -13,7 +13,6 @@ Usage:
 from __future__ import annotations
 
 from collections.abc import Iterator, Sequence
-from typing import Optional
 
 BINOPS_EQNE = (("__eq__", "nputils.array_eq"), ("__ne__", "nputils.array_ne"))
 BINOPS_CMP = (
@@ -88,12 +87,10 @@ template_binop = """
         return self._binary_op(other, {func})"""
 template_binop_overload = """
     @overload{overload_type_ignore}
-    def {method}(self, other: {overload_type}) -> {overload_type}:
-        ...
+    def {method}(self, other: {overload_type}) -> {overload_type}: ...
 
     @overload
-    def {method}(self, other: {other_type}) -> {return_type}:
-        ...
+    def {method}(self, other: {other_type}) -> {return_type}: ...
 
     def {method}(self, other: {other_type}) -> {return_type} | {overload_type}:{type_ignore}
         return self._binary_op(other, {func})"""
@@ -129,7 +126,7 @@ unhashable = """
 # The type ignores might not be necessary anymore at some point.
 #
 # We require a "hack" to tell type checkers that e.g. Variable + DataArray = DataArray
-# In reality this returns NotImplementes, but this is not a valid type in python 3.9.
+# In reality this returns NotImplemented, but this is not a valid type in python 3.9.
 # Therefore, we return DataArray. In reality this would call DataArray.__add__(Variable)
 # TODO: change once python 3.10 is the minimum.
 #
@@ -141,7 +138,7 @@ def _type_ignore(ignore: str) -> str:
     return f"  # type:ignore[{ignore}]" if ignore else ""
 
 
-FuncType = Sequence[tuple[Optional[str], Optional[str]]]
+FuncType = Sequence[tuple[str | None, str | None]]
 OpsType = tuple[FuncType, str, dict[str, str]]
 
 
@@ -216,7 +213,13 @@ def unops() -> list[OpsType]:
     ]
 
 
+# We use short names T_DA and T_DS to keep below 88 lines so
+# ruff does not reformat everything. When reformatting, the
+# type-ignores end up in the wrong line :/
+
 ops_info = {}
+# TODO add inplace ops for DataTree?
+ops_info["DataTreeOpsMixin"] = binops(other_type="DtCompatible") + unops()
 ops_info["DatasetOpsMixin"] = (
     binops(other_type="DsCompatible") + inplace(other_type="DsCompatible") + unops()
 )
@@ -224,12 +227,12 @@ ops_info["DataArrayOpsMixin"] = (
     binops(other_type="DaCompatible") + inplace(other_type="DaCompatible") + unops()
 )
 ops_info["VariableOpsMixin"] = (
-    binops_overload(other_type="VarCompatible", overload_type="T_DataArray")
+    binops_overload(other_type="VarCompatible", overload_type="T_DA")
     + inplace(other_type="VarCompatible", type_ignore="misc")
     + unops()
 )
 ops_info["DatasetGroupByOpsMixin"] = binops(
-    other_type="GroupByCompatible", return_type="Dataset"
+    other_type="Dataset | DataArray", return_type="Dataset"
 )
 ops_info["DataArrayGroupByOpsMixin"] = binops(
     other_type="T_Xarray", return_type="T_Xarray"
@@ -237,26 +240,29 @@ ops_info["DataArrayGroupByOpsMixin"] = binops(
 
 MODULE_PREAMBLE = '''\
 """Mixin classes with arithmetic operators."""
+
 # This file was generated using xarray.util.generate_ops. Do not edit manually.
 
 from __future__ import annotations
 
 import operator
-from typing import TYPE_CHECKING, Any, Callable, overload
+from collections.abc import Callable
+from typing import TYPE_CHECKING, Any, overload
 
 from xarray.core import nputils, ops
 from xarray.core.types import (
     DaCompatible,
     DsCompatible,
-    GroupByCompatible,
+    DtCompatible,
     Self,
-    T_DataArray,
     T_Xarray,
     VarCompatible,
 )
 
 if TYPE_CHECKING:
-    from xarray.core.dataset import Dataset'''
+    from xarray.core.dataarray import DataArray
+    from xarray.core.dataset import Dataset
+    from xarray.core.types import T_DataArray as T_DA'''
 
 
 CLASS_PREAMBLE = """{newline}
